@@ -5,15 +5,10 @@ Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 """
 
 import sys
-import os
 
 if sys.version_info < (2, 6):
     import warnings
-    if os.name == "java":
-        if sys.version_info < (2, 5):
-            warnings.warn("This Pyro version is unsupported on Jython versions older than 2.5", ImportWarning)
-    else:
-        warnings.warn("This Pyro version is unsupported on Python versions older than 2.6", ImportWarning)
+    warnings.warn("This Pyro version is unsupported on Python versions older than 2.6", ImportWarning)
 
 
 def _configLogging():
@@ -24,7 +19,8 @@ def _configLogging():
     Set the env var PYRO_LOGFILE to change the name of the autoconfigured
     log file (default is pyro.log in the current dir). Use '{stderr}' to
     make the log go to the standard error output."""
-    import os, logging
+    import os
+    import logging
     level = os.environ.get("PYRO_LOGLEVEL")
     logfilename = os.environ.get("PYRO_LOGFILE", "pyro.log")
     if logfilename == "{stderr}":
@@ -60,7 +56,7 @@ del _configLogging
 
 # initialize Pyro's configuration
 from Pyro4.configuration import Configuration
-config=Configuration()
+config = Configuration()
 del Configuration
 
 # import the required Pyro symbols into this package
