@@ -12,6 +12,7 @@ except ImportError:
     import copy_reg as copyreg
 import Pyro4
 import Pyro4.errors
+import Pyro4.message
 
 log=logging.getLogger("Pyro4.util")
 
@@ -139,6 +140,7 @@ for name, t in vars(Pyro4.errors).items():
 class SerializerBase(object):
     """Base class for (de)serializer implementations (which must be thread safe)"""
     __custom_class_to_dict_registry = {}
+    __custom_dict_to_class_registry = {}
 
     def serializeData(self, data, compress=False):
         """Serialize the given data object, try to compress if told so.
@@ -190,13 +192,46 @@ class SerializerBase(object):
         raise NotImplementedError("implement in subclass")
 
     @classmethod
-    def register_class_to_dict(cls, clazz, converter):
+    def register_class_to_dict(cls, clazz, converter, serpent_too=True):
+        """Registers a custom function that returns a dict representation of objects of the given class.
+        The function is called with a single parameter; the object to be converted to a dict."""
         cls.__custom_class_to_dict_registry[clazz] = converter
+        if serpent_too:
+            try:
+                get_serializer_by_id(SerpentSerializer.serializer_id)
+                import serpent
+                def serpent_converter(obj, serializer, stream, level):
+                    d = converter(obj)
+                    serializer.ser_builtins_dict(d, stream, level)
+                serpent.register_class(clazz, serpent_converter)
+            except Pyro4.errors.ProtocolError:
+                pass
 
     @classmethod
     def unregister_class_to_dict(cls, clazz):
+        """Removes the to-dict conversion function registered for the given class. Objects of the class
+        will be serialized by the default mechanism again."""
         if clazz in cls.__custom_class_to_dict_registry:
             del cls.__custom_class_to_dict_registry[clazz]
+        try:
+            get_serializer_by_id(SerpentSerializer.serializer_id)
+            import serpent
+            serpent.unregister_class(clazz)
+        except Pyro4.errors.ProtocolError:
+            pass
+
+    @classmethod
+    def register_dict_to_class(cls, classname, converter):
+        """Registers a custom converter function that creates objects from a dict with the given classname tag in it.
+        The function is called with two parameters: the classname and the dictionary to convert to an instance of the class."""
+        cls.__custom_dict_to_class_registry[classname] = converter
+
+    @classmethod
+    def unregister_dict_to_class(cls, classname):
+        """Removes the converter registered for the given classname. Dicts with that classname tag
+        will be deserialized by the default mechanism again."""
+        if classname in cls.__custom_dict_to_class_registry:
+            del cls.__custom_dict_to_class_registry[classname]
 
     @classmethod
     def class_to_dict(cls, obj):
@@ -210,10 +245,12 @@ class SerializerBase(object):
             obj._pyroDaemon = None
         if isinstance(obj, BaseException):
             # special case for exceptions
-            value={"args": obj.args}
-            value["__class__"] = obj.__class__.__module__ + "." + obj.__class__.__name__
-            value["attributes"] = vars(obj)  # add custom exception attributes
-            return value
+            return {
+                "__class__": obj.__class__.__module__ + "." + obj.__class__.__name__,
+                "__exception__": True,
+                "args": obj.args,
+                "attributes": vars(obj)  # add custom exception attributes
+            }
         try:
             value = obj.__getstate__()
         except AttributeError:
@@ -236,38 +273,43 @@ class SerializerBase(object):
             else:
                 raise Pyro4.errors.ProtocolError("don't know how to serialize class " + str(obj.__class__) + ". Give it vars() or an appropriate __getstate__")
 
-    @staticmethod
-    def dict_to_class(data):
+    @classmethod
+    def dict_to_class(cls, data):
         """
         Recreate an object out of a dict containing the class name and the attributes.
         Only a fixed set of classes are recognized.
         """
         classname = data.get("__class__", "<unknown>")
+        if isinstance(classname, bytes):
+            classname = classname.decode("utf-8")
+
+        if classname in cls.__custom_dict_to_class_registry:
+            converter = cls.__custom_dict_to_class_registry[classname]
+            return converter(classname, data)
         if "__" in classname:
             raise Pyro4.errors.SecurityError("refused to deserialize types with double underscores in their name: "+classname)
+        # because of efficiency reasons the constructors below are hardcoded here instead of added on a per-class basis to the dict-to-class registry
         if classname.startswith("Pyro4.core."):
-            if classname=="Pyro4.core.URI":
+            if classname == "Pyro4.core.URI":
                 uri = Pyro4.core.URI.__new__(Pyro4.core.URI)
-                uri.__setstate__(data["state"])
+                uri.__setstate_from_dict__(data["state"])
                 return uri
-            elif classname=="Pyro4.core.Proxy":
+            elif classname == "Pyro4.core.Proxy":
                 proxy = Pyro4.core.Proxy.__new__(Pyro4.core.Proxy)
-                state = data["state"]
-                uri = Pyro4.core.URI(state[0])
-                oneway = set(state[1])
-                timeout = state[2]
-                proxy.__setstate__((uri, oneway, timeout))
+                proxy.__setstate_from_dict__(data["state"])
                 return proxy
-            elif classname=="Pyro4.core.Daemon":
-                return Pyro4.core.Daemon.__new__(Pyro4.core.Daemon)
+            elif classname == "Pyro4.core.Daemon":
+                daemon = Pyro4.core.Daemon.__new__(Pyro4.core.Daemon)
+                daemon.__setstate_from_dict__(data["state"])
+                return daemon
         elif classname.startswith("Pyro4.util."):
-            if classname=="Pyro4.util.PickleSerializer":
+            if classname == "Pyro4.util.PickleSerializer":
                 return PickleSerializer()
-            elif classname=="Pyro4.util.MarshalSerializer":
+            elif classname == "Pyro4.util.MarshalSerializer":
                 return MarshalSerializer()
-            elif classname=="Pyro4.util.JsonSerializer":
+            elif classname == "Pyro4.util.JsonSerializer":
                 return JsonSerializer()
-            elif classname=="Pyro4.util.SerpentSerializer":
+            elif classname == "Pyro4.util.SerpentSerializer":
                 return SerpentSerializer()
         elif classname.startswith("Pyro4.errors."):
             errortype = getattr(Pyro4.errors, classname.split('.', 2)[2])
@@ -276,16 +318,22 @@ class SerializerBase(object):
         elif classname == "Pyro4.futures._ExceptionWrapper":
             ex = SerializerBase.dict_to_class(data["exception"])
             return Pyro4.futures._ExceptionWrapper(ex)
-        elif classname.startswith("builtins."):
-            exceptiontype = getattr(builtins, classname.split('.', 1)[1])
-            if issubclass(exceptiontype, BaseException):
-                return SerializerBase.make_exception(exceptiontype, data)
-        elif classname.startswith("exceptions."):
-            exceptiontype = getattr(exceptions, classname.split('.', 1)[1])
-            if issubclass(exceptiontype, BaseException):
-                return SerializerBase.make_exception(exceptiontype, data)
-        elif classname in all_exceptions:
-            return SerializerBase.make_exception(all_exceptions[classname], data)
+        elif data.get("__exception__", False):
+            if classname in all_exceptions:
+                return SerializerBase.make_exception(all_exceptions[classname], data)
+            # python 2.x: exceptions.ValueError
+            # python 3.x: builtins.ValueError
+            # translate to the appropriate namespace...
+            namespace, short_classname = classname.split('.', 1)
+            if namespace in ("builtins", "exceptions"):
+                if sys.version_info < (3, 0):
+                    exceptiontype = getattr(exceptions, short_classname)
+                    if issubclass(exceptiontype, BaseException):
+                        return SerializerBase.make_exception(exceptiontype, data)
+                else:
+                    exceptiontype = getattr(builtins, short_classname)
+                    if issubclass(exceptiontype, BaseException):
+                        return SerializerBase.make_exception(exceptiontype, data)
         # try one of the serializer classes
         for serializer in _serializers.values():
             if classname == serializer.__class__.__name__:
@@ -332,11 +380,13 @@ class PickleSerializer(SerializerBase):
     A (de)serializer that wraps the Pickle serialization protocol.
     It can optionally compress the serialized data, and is thread safe.
     """
+    serializer_id = Pyro4.message.SERIALIZER_PICKLE
+
     def dumpsCall(self, obj, method, vargs, kwargs):
-        return pickle.dumps((obj, method, vargs, kwargs), pickle.HIGHEST_PROTOCOL)
+        return pickle.dumps((obj, method, vargs, kwargs), Pyro4.config.PICKLE_PROTOCOL_VERSION)
 
     def dumps(self, data):
-        return pickle.dumps(data, pickle.HIGHEST_PROTOCOL)
+        return pickle.dumps(data, Pyro4.config.PICKLE_PROTOCOL_VERSION)
 
     def loadsCall(self, data):
         return pickle.loads(data)
@@ -348,7 +398,6 @@ class PickleSerializer(SerializerBase):
     def register_type_replacement(cls, object_type, replacement_function):
         def copyreg_function(obj):
             return replacement_function(obj).__reduce__()
-
         try:
             copyreg.pickle(object_type, copyreg_function)
         except TypeError:
@@ -357,6 +406,8 @@ class PickleSerializer(SerializerBase):
 
 class MarshalSerializer(SerializerBase):
     """(de)serializer that wraps the marshal serialization protocol."""
+    serializer_id = Pyro4.message.SERIALIZER_MARSHAL
+
     def dumpsCall(self, obj, method, vargs, kwargs):
         return marshal.dumps((obj, method, vargs, kwargs))
 
@@ -382,11 +433,13 @@ class MarshalSerializer(SerializerBase):
 
 class SerpentSerializer(SerializerBase):
     """(de)serializer that wraps the serpent serialization protocol."""
+    serializer_id = Pyro4.message.SERIALIZER_SERPENT
+
     def dumpsCall(self, obj, method, vargs, kwargs):
-        return serpent.dumps((obj, method, vargs, kwargs))
+        return serpent.dumps((obj, method, vargs, kwargs), module_in_classname=True)
 
     def dumps(self, data):
-        return serpent.dumps(data)
+        return serpent.dumps(data, module_in_classname=True)
 
     def loadsCall(self, data):
         obj, method, vargs, kwargs = serpent.loads(data)
@@ -410,6 +463,8 @@ class SerpentSerializer(SerializerBase):
 
 class JsonSerializer(SerializerBase):
     """(de)serializer that wraps the json serialization protocol."""
+    serializer_id = Pyro4.message.SERIALIZER_JSON
+
     __type_replacements = {}
     def dumpsCall(self, obj, method, vargs, kwargs):
         data = {"object": obj, "method": method, "params": vargs, "kwargs": kwargs}
@@ -419,13 +474,13 @@ class JsonSerializer(SerializerBase):
         data = json.dumps(data, ensure_ascii=False, default=self.default)
         return data.encode("utf-8")
     def loadsCall(self, data):
-        data=data.decode("utf-8")
+        data = data.decode("utf-8")
         data = json.loads(data)
         vargs = self.recreate_classes(data["params"])
         kwargs = self.recreate_classes(data["kwargs"])
         return data["object"], data["method"], vargs, kwargs
     def loads(self, data):
-        data=data.decode("utf-8")
+        data = data.decode("utf-8")
         return self.recreate_classes(json.loads(data))
     def default(self, obj):
         replacer = self.__type_replacements.get(type(obj), None)
@@ -439,33 +494,55 @@ class JsonSerializer(SerializerBase):
 
 """The various serializers that are supported"""
 _serializers = {}
-def get_serializer():
+_serializers_by_id = {}
+def get_serializer(name):
     try:
-        return _serializers[Pyro4.config.SERIALIZER]
+        return _serializers[name]
     except KeyError:
-        raise Pyro4.errors.ProtocolError("configured serializer '%s' is unknown or not available" % Pyro4.config.SERIALIZER)
+        raise Pyro4.errors.ProtocolError("serializer '%s' is unknown or not available" % name)
+
+def get_serializer_by_id(sid):
+    try:
+        return _serializers_by_id[sid]
+    except KeyError:
+        raise Pyro4.errors.ProtocolError("no serializer available for id %d" % sid)
 
 # determine the serializers that are supported
 try:
     import cPickle as pickle
 except ImportError:
     import pickle
-if pickle.HIGHEST_PROTOCOL<2:
-    raise RuntimeError("pickle serializer needs to support protocol 2 or higher")
-_serializers["pickle"] = PickleSerializer()
+assert Pyro4.config.PICKLE_PROTOCOL_VERSION>=2, "pickle protocol needs to be 2 or higher"
+_ser = PickleSerializer()
+_serializers["pickle"] = _ser
+_serializers_by_id[_ser.serializer_id] = _ser
 import marshal
-_serializers["marshal"] = MarshalSerializer()
+_ser = MarshalSerializer()
+_serializers["marshal"] = _ser
+_serializers_by_id[_ser.serializer_id] = _ser
 try:
     import json
-    _serializers["json"] = JsonSerializer()
+    _ser = JsonSerializer()
+    _serializers["json"] = _ser
+    _serializers_by_id[_ser.serializer_id] = _ser
 except ImportError:
     pass
 try:
     import serpent
-    _serializers["serpent"] = SerpentSerializer()
+    if '-' in serpent.__version__:
+        ver = serpent.__version__.split('-', 1)[0]
+    else:
+        ver = serpent.__version__
+    ver = tuple(map(int, ver.split(".")))
+    if ver<(1, 5):
+        raise RuntimeError("requires serpent 1.5 or better")
+    _ser = SerpentSerializer()
+    _serializers["serpent"] = _ser
+    _serializers_by_id[_ser.serializer_id] = _ser
 except ImportError:
-    #warnings.warn("serpent serializer not available", RuntimeWarning)
+    # warnings.warn("serpent serializer not available", RuntimeWarning)
     pass
+del _ser
 
 
 def resolveDottedAttribute(obj, attr, allowDotted):
@@ -494,8 +571,11 @@ def excepthook(ex_type, ex_value, ex_tb):
 
 
 def fixIronPythonExceptionForPickle(exceptionObject, addAttributes):
-    """function to hack around a bug in IronPython where it doesn't pickle
-    exception attributes. We piggyback them into the exception's args."""
+    """
+    Function to hack around a bug in IronPython where it doesn't pickle
+    exception attributes. We piggyback them into the exception's args.
+    Bug report is at http://ironpython.codeplex.com/workitem/30805
+    """
     if hasattr(exceptionObject, "args"):
         if addAttributes:
             # piggyback the attributes on the exception args instead.
