@@ -6,11 +6,15 @@ Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 
 from __future__ import with_statement
 import sys
+import os
+import copy
 import pprint
+import pickle
 import Pyro4.util
 import Pyro4.errors
 import Pyro4.core
 import Pyro4.futures
+import Pyro4.message
 from testsupport import *
 
 
@@ -20,13 +24,13 @@ class SerializeTests_pickle(unittest.TestCase):
         self.previous_serializer=Pyro4.config.SERIALIZER
         Pyro4.config.SERIALIZER=self.SERIALIZER
         Pyro4.config.HMAC_KEY=b"testsuite"
-        self.ser=Pyro4.util.get_serializer()
+        self.ser=Pyro4.util.get_serializer(Pyro4.config.SERIALIZER)
     def tearDown(self):
         Pyro4.config.HMAC_KEY=None
         Pyro4.config.SERIALIZER=self.previous_serializer
         
     def testSerItself(self):
-        s=Pyro4.util.get_serializer()
+        s=Pyro4.util.get_serializer(Pyro4.config.SERIALIZER)
         p,_=self.ser.serializeData(s)
         s2=self.ser.deserializeData(p)
         self.assertEqual(s,s2)
@@ -36,9 +40,10 @@ class SerializeTests_pickle(unittest.TestCase):
     def testSerUnicode(self):
         data = unicode("x")
         ser,_ = self.ser.serializeData(data)
-        self.assertTrue(type(ser) is bytes)
+        expected_type = str if sys.platform=="cli" else bytes   # ironpython serializes into str, not bytes... :(
+        self.assertTrue(type(ser) is expected_type)
         ser,_ = self.ser.serializeCall(data, unicode("method"), [], {})
-        self.assertTrue(type(ser) is bytes)
+        self.assertTrue(type(ser) is expected_type)
 
     def testSerCompression(self):
         d1,c1=self.ser.serializeData("small data", compress=True)
@@ -57,12 +62,17 @@ class SerializeTests_pickle(unittest.TestCase):
     def testSerErrors(self):
         e1=Pyro4.errors.NamingError(unicode("x"))
         e1._pyroTraceback = ["this is the remote traceback"]
+        orig_e = copy.copy(e1)
         e2=Pyro4.errors.PyroError(unicode("x"))
         e3=Pyro4.errors.ProtocolError(unicode("x"))
+        if sys.platform=="cli":
+            Pyro4.util.fixIronPythonExceptionForPickle(e1, True)
         p,_=self.ser.serializeData(e1)
         e=self.ser.deserializeData(p)
+        if sys.platform=="cli":
+            Pyro4.util.fixIronPythonExceptionForPickle(e, False)
         self.assertTrue(isinstance(e, Pyro4.errors.NamingError))
-        self.assertEqual(repr(e1), repr(e))
+        self.assertEqual(repr(orig_e), repr(e))
         self.assertEqual(["this is the remote traceback"], e._pyroTraceback, "remote traceback info should be present")
         p,_=self.ser.serializeData(e2)
         e=self.ser.deserializeData(p)
@@ -113,7 +123,6 @@ class SerializeTests_pickle(unittest.TestCase):
         self.assertTrue(proxy._pyroConnection is None)
         self.assertTrue(proxy2._pyroConnection is None)
         self.assertEqual(proxy2._pyroUri, proxy._pyroUri)
-        self.assertEqual(proxy2._pyroSerializer, proxy._pyroSerializer)
         self.assertEqual(42, proxy2._pyroTimeout)
 
     def testNested(self):
@@ -154,25 +163,29 @@ class SerializeTests_pickle(unittest.TestCase):
         uri = Pyro4.core.URI("PYRO:object@host:4444")
         s, c = self.ser.serializeData(uri)
         x = self.ser.deserializeData(s, c)
+        self.assertIsInstance(x, Pyro4.core.URI)
         self.assertEqual(uri, x)
         uri=Pyro4.core.URI("PYRO:12345@./u:/tmp/socketname")
         s, c = self.ser.serializeData(uri)
         x = self.ser.deserializeData(s, c)
+        self.assertIsInstance(x, Pyro4.core.URI)
         self.assertEqual(uri, x)
         proxy=Pyro4.core.Proxy(uri)
         s, c = self.ser.serializeData(proxy)
         x = self.ser.deserializeData(s, c)
+        self.assertIsInstance(x, Pyro4.core.Proxy)
         self.assertEqual(proxy._pyroUri, x._pyroUri)
         self.assertTrue(x._)
         daemon=Pyro4.core.Daemon()
         s, c = self.ser.serializeData(daemon)
         x = self.ser.deserializeData(s, c)
+        self.assertIsInstance(x, Pyro4.core.Daemon)
         self.assertTrue(isinstance(x, Pyro4.core.Daemon))
         wrapper = Pyro4.futures._ExceptionWrapper(ZeroDivisionError("divided by zero"))
         s, c = self.ser.serializeData(wrapper)
         x = self.ser.deserializeData(s, c)
         self.assertIsInstance(x, Pyro4.futures._ExceptionWrapper)
-        self.assertEquals("divided by zero", str(x.exception))
+        self.assertEqual("divided by zero", str(x.exception))
 
     def testAutoProxy(self):
         if self.SERIALIZER=="marshal":
@@ -205,10 +218,39 @@ class SerializeTests_pickle(unittest.TestCase):
         except Pyro4.errors.ProtocolError:
             pass
 
+    def testCustomClassOk(self):
+        if self.SERIALIZER=="pickle":
+            self.skipTest("pickle simply serializes custom classes just fine")
+        o = MyThing2("test")
+        Pyro4.util.SerializerBase.register_class_to_dict(MyThing2, mything_dict)
+        Pyro4.util.SerializerBase.register_dict_to_class("CUSTOM-Mythingymabob", mything_creator)
+        s, c = self.ser.serializeData(o)
+        o2 = self.ser.deserializeData(s, c)
+        self.assertIsInstance(o2, MyThing2)
+        self.assertEqual("test", o2.name)
+        # unregister the deserializer
+        Pyro4.util.SerializerBase.unregister_dict_to_class("CUSTOM-Mythingymabob")
+        try:
+            self.ser.deserializeData(s, c)
+            self.fail("must fail")
+        except Pyro4.errors.ProtocolError:
+            pass  # ok
+        # unregister the serializer
+        Pyro4.util.SerializerBase.unregister_class_to_dict(MyThing2)
+        s, c = self.ser.serializeData(o)
+        try:
+            self.ser.deserializeData(s, c)
+            self.fail("must fail")
+        except Pyro4.errors.ProtocolError as x:
+            msg = str(x)
+            self.assertTrue(msg in ["unsupported serialized class: testsupport.MyThing2",
+                                    "unsupported serialized class: PyroTests.testsupport.MyThing2"])
+
     def testData(self):
         data = [42, "hello"]
         ser, compressed = self.ser.serializeData(data)
-        self.assertTrue(type(ser) is bytes)
+        expected_type = str if sys.platform=="cli" else bytes   # ironpython serializes into str, not bytes... :(
+        self.assertTrue(type(ser) is expected_type)
         self.assertFalse(compressed)
         data2 = self.ser.deserializeData(ser, compressed=False)
         self.assertEqual(data, data2)
@@ -260,11 +302,20 @@ class SerializeTests_pickle(unittest.TestCase):
         e2 = self.ser.deserializeData(d, c)
         self.assertTrue(isinstance(e2, ZeroDivisionError))
         self.assertEqual("hello", str(e2))
+        e = ZeroDivisionError("hello", 42)
+        d, c = self.ser.serializeData(e)
+        e2 = self.ser.deserializeData(d, c)
+        self.assertTrue(isinstance(e2, ZeroDivisionError))
+        self.assertTrue(str(e2) in ("('hello', 42)", "(u'hello', 42)"))
         e.custom_attribute = 999
+        if sys.platform=="cli":
+            Pyro4.util.fixIronPythonExceptionForPickle(e, True)
         ser, compressed = self.ser.serializeData(e)
         e2 = self.ser.deserializeData(ser, compressed)
+        if sys.platform=="cli":
+            Pyro4.util.fixIronPythonExceptionForPickle(e2, False)
         self.assertTrue(isinstance(e2, ZeroDivisionError))
-        self.assertEqual("hello", str(e2))
+        self.assertTrue(str(e2) in ("('hello', 42)", "(u'hello', 42)"))
         self.assertEqual(999, e2.custom_attribute)
 
     def testSerializeSpecialException(self):
@@ -289,40 +340,51 @@ class SerializeTests_pickle(unittest.TestCase):
         self.assertEqual(1, number)
         self.assertEqual(Pyro4.core.URI("PYRO:555@localhost:80"), uri["uri"])
 
+    def testProtocolVersion(self):
+        self.assertGreaterEqual(Pyro4.config.PICKLE_PROTOCOL_VERSION, 2)
+        self.assertEqual(pickle.HIGHEST_PROTOCOL, Pyro4.config.PICKLE_PROTOCOL_VERSION)
+
 
 class SerializeTests_serpent(SerializeTests_pickle):
-    SERIALIZER="serpent"
+    SERIALIZER = "serpent"
+    @unittest.skip("pickle specific")
+    def testProtocolVersion(self):
+        pass
 
 class SerializeTests_json(SerializeTests_pickle):
-    SERIALIZER="json"
+    SERIALIZER = "json"
+    @unittest.skip("pickle specific")
+    def testProtocolVersion(self):
+        pass
 
-class SerializeTests_marshal(SerializeTests_pickle):
-    SERIALIZER="marshal"
 
+if os.name!="java":
+    # The marshal serializer is not working correctly under jython,
+    # see http://bugs.jython.org/issue2077
+    # So we only include this when not running jython
+    class SerializeTests_marshal(SerializeTests_pickle):
+        SERIALIZER="marshal"
 
 
 class GenericTests(unittest.TestCase):
     def testSerializersAvailable(self):
-        previous_serializer=Pyro4.config.SERIALIZER
+        Pyro4.util.get_serializer("pickle")
+        Pyro4.util.get_serializer("marshal")
         try:
-            Pyro4.config.SERIALIZER="pickle"
-            Pyro4.util.get_serializer()
-            Pyro4.config.SERIALIZER="marshal"
-            Pyro4.util.get_serializer()
-            try:
-                import json
-                Pyro4.config.SERIALIZER="json"
-                Pyro4.util.get_serializer()
-            except ImportError:
-                pass
-            try:
-                import serpent
-                Pyro4.config.SERIALIZER="serpent"
-                Pyro4.util.get_serializer()
-            except ImportError:
-                pass
-        finally:
-            Pyro4.config.SERIALIZER=previous_serializer
+            import json
+            Pyro4.util.get_serializer("json")
+        except ImportError:
+            pass
+        try:
+            import serpent
+            Pyro4.util.get_serializer("serpent")
+        except ImportError:
+            pass
+
+    def testSerializersAvailableById(self):
+        Pyro4.util.get_serializer_by_id(Pyro4.message.SERIALIZER_PICKLE)
+        Pyro4.util.get_serializer_by_id(Pyro4.message.SERIALIZER_MARSHAL)
+        self.assertRaises(Pyro4.errors.ProtocolError, lambda: Pyro4.util.get_serializer_by_id(9999999))
 
     def testDictClassFail(self):
         o = pprint.PrettyPrinter(stream="dummy", width=42)
@@ -330,7 +392,7 @@ class GenericTests(unittest.TestCase):
         self.assertEqual(42, d["_width"])
         self.assertEqual("pprint.PrettyPrinter", d["__class__"])
         try:
-            x = Pyro4.util.SerializerBase.dict_to_class(d)
+            _ = Pyro4.util.SerializerBase.dict_to_class(d)
             self.fail("error expected")
         except Pyro4.errors.ProtocolError:
             pass
@@ -339,6 +401,7 @@ class GenericTests(unittest.TestCase):
         x = ZeroDivisionError("hello", 42)
         expected = {
             "__class__": None,
+            "__exception__": True,
             "args": ("hello", 42),
             "attributes": {}
         }
@@ -359,6 +422,7 @@ class GenericTests(unittest.TestCase):
         self.assertEqual("Pyro4.core.URI", d["__class__"])
         self.assertTrue("state" in d)
         x = Pyro4.util.SerializerBase.dict_to_class(d)
+        self.assertIsInstance(x, Pyro4.core.URI)
         self.assertEqual(uri, x)
         self.assertEqual(4444, x.port)
         uri = Pyro4.core.URI("PYRO:12345@./u:/tmp/socketname")
@@ -366,9 +430,73 @@ class GenericTests(unittest.TestCase):
         self.assertEqual("Pyro4.core.URI", d["__class__"])
         self.assertTrue("state" in d)
         x = Pyro4.util.SerializerBase.dict_to_class(d)
+        self.assertIsInstance(x, Pyro4.core.URI)
         self.assertEqual(uri, x)
         self.assertEqual("/tmp/socketname", x.sockname)
 
+    def testCustomDictClass(self):
+        o = MyThing2("test")
+        Pyro4.util.SerializerBase.register_class_to_dict(MyThing2, mything_dict)
+        Pyro4.util.SerializerBase.register_dict_to_class("CUSTOM-Mythingymabob", mything_creator)
+        d = Pyro4.util.SerializerBase.class_to_dict(o)
+        self.assertEqual("CUSTOM-Mythingymabob", d["__class__"])
+        self.assertEqual("test", d["name"])
+        x = Pyro4.util.SerializerBase.dict_to_class(d)
+        self.assertIsInstance(x, MyThing2)
+        self.assertEqual("test", x.name)
+        # unregister the conversion functions and try again
+        Pyro4.util.SerializerBase.unregister_class_to_dict(MyThing2)
+        Pyro4.util.SerializerBase.unregister_dict_to_class("CUSTOM-Mythingymabob")
+        d_orig = Pyro4.util.SerializerBase.class_to_dict(o)
+        clsname = d_orig["__class__"]
+        self.assertTrue(clsname.endswith("testsupport.MyThing2"))
+        try:
+            _ = Pyro4.util.SerializerBase.dict_to_class(d)
+            self.fail("should crash")
+        except Pyro4.errors.ProtocolError:
+            pass  # ok
+
+    def testExceptionNamespacePy2(self):
+        data = {'__class__': 'exceptions.ZeroDivisionError',
+                '__exception__': True,
+                'args': ('hello', 42),
+                'attributes': {"test_attribute": 99}}
+        exc = Pyro4.util.SerializerBase.dict_to_class(data)
+        self.assertIsInstance(exc, ZeroDivisionError)
+        self.assertEqual("ZeroDivisionError('hello', 42)", repr(exc))
+        self.assertEqual(99, exc.test_attribute)
+
+    def testExceptionNamespacePy3(self):
+        data = {'__class__': 'builtins.ZeroDivisionError',
+                '__exception__': True,
+                'args': ('hello', 42),
+                'attributes': {"test_attribute": 99}}
+        exc = Pyro4.util.SerializerBase.dict_to_class(data)
+        self.assertIsInstance(exc, ZeroDivisionError)
+        self.assertEqual("ZeroDivisionError('hello', 42)", repr(exc))
+        self.assertEqual(99, exc.test_attribute)
+
+    def testExceptionNotTagged(self):
+        data = {'__class__': 'builtins.ZeroDivisionError',
+                'args': ('hello', 42),
+                'attributes': {}}
+        with self.assertRaises(Pyro4.errors.ProtocolError) as cm:
+            _ = Pyro4.util.SerializerBase.dict_to_class(data)
+        self.assertEqual("unsupported serialized class: builtins.ZeroDivisionError", str(cm.exception))
+
+
+def mything_dict(obj):
+    return {
+        "__class__": "CUSTOM-Mythingymabob",
+        "name": obj.name
+    }
+
+
+def mything_creator(classname, d):
+    assert classname == "CUSTOM-Mythingymabob"
+    assert d["__class__"] == "CUSTOM-Mythingymabob"
+    return MyThing2(d["name"])
+
 
 if __name__ == "__main__":
     #import sys;sys.argv = ['', 'Test.testName']
